/*
 * This software is Copyright (C) 2006-2008
 * Fachhochschule Hannover (University of Applied Sciences and Arts)
 * Use is subject to license conditions.
 *
 * The main licensing options available are:
 *
 * Open Source Licensing. This is the appropriate option if you want to
 * share the source code of your application with everyone you
 * distribute it to, and you also want to give them the right to share
 * who uses it. If you wish to use TNC@FHH under Open Source Licensing,
 * you must contribute all your source code to the open source
 * community in accordance with the GPL Version 2 when your application
 * is distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * Commercial Licensing. This is the appropriate option if you are
 * creating proprietary applications and you are not prepared to
 * distribute and share the source code of your application.
 * Contact tnc@inform.fh-hannover.de for details.
 *
 * http://tnc.inform.fh-hannover.de/
 */

#include "IMVHostScanner.h"
/* TNCUtil */
#include <TNCUtil/io/TNCLog.h>
#include <TNCUtil/io/TNCIOHelper.h>
#include <TNCUtil/tcg/tnc/tncifimv.h>
#include <TNCUtil/TNCUtil.h>
#include <TNCUtil/exception/Exception.h>
#include <errno.h>
#include <stdlib.h>

#define CLASSNAME "IMVHostScanner"

#define THROW_ERROR(x) throw TNCUtil::exception::Exception("error in %s:%d %s",__FILE__,__LINE__,x);

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *             IMVHostScanner::IMVHostScanner(TNC_ConnectionID)              *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
IMVHostScanner::IMVHostScanner(TNC_ConnectionID conID)
    : TNCUtil::iml::AbstractIMV(conID), count(0)
{
    TNCLog::getLog(CLASSNAME, "Constructor");
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                    IMVHostScanner::~IMVHostScanner()                      *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
IMVHostScanner::~IMVHostScanner() {
    TNCLog::getLog(CLASSNAME, "~");
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *            TNC_Result IMVHostScanner::notifyConnectionChange()            *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result IMVHostScanner::notifyConnectionChange()
{
	if (this->getConnectionState() == TNC_CONNECTION_STATE_HANDSHAKE) {

		this->policys.clear();
	    this->count = 0;


	    std::ifstream * policy = TNCUtil::io::openFile(POLICY_FILE);
	    if (!policy)
	    	THROW_ERROR("policy file not found");

	    std::vector<std::pair<std::string,std::string> > properties = TNCUtil::io::readAllProperties(policy);

	    TNCUtil::io::closeFile(policy);
	    policy = NULL;

		for (std::vector<std::pair<std::string,std::string> >::iterator iter = properties.begin();
		    iter != properties.end();
		    ++iter) {

			/* read Protocol */
			Protocol protocol;
			if (!iter->first.compare(0, 3, "TCP"))
				protocol = TCP;
			else if (!iter->first.compare(0, 3, "UDP"))
				protocol = UDP;
			else
				THROW_ERROR("Protocol is unknown (not TCP or UDP)");

			/* read Port */
			size_t tmpPos = iter->first.find_first_of("1234567890", 3);
			if (tmpPos==std::string::npos)
				THROW_ERROR("No Portnumber found");
			char * end = NULL;
			unsigned long int port = strtoul(iter->first.substr(tmpPos, iter->first.size()).c_str(), &end, 10);
			if (errno==ERANGE || errno==EINVAL || *end)
				THROW_ERROR("IMVHostScanner(): Error by string to unsigned int.");

			/* read Status */
			Status status;
			if (!iter->second.compare("open"))
				status = open;
			else if (!iter->second.compare("close"))
				status = close;
			else if (!iter->second.compare("whatever"))
				status = whatever;
			else
				THROW_ERROR("Status is unknown (not open, close or whatever)");

			this->policys.push_back(Policy(protocol, (unsigned short) port, status));
		}
	}

	return TNC_RESULT_SUCCESS;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 * TNC_Result IMVHostScanner::receiveMessage(TNC_BufferReference message,    *
 *            TNC_UInt32 messageLength, TNC_MessageType messageType)         *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result IMVHostScanner::receiveMessage(TNC_BufferReference message, TNC_UInt32 messageLength, TNC_MessageType messageType)
{
    TNCLog &log = TNCLog::getLog(CLASSNAME, "receiveMessage");
    TNC_Result ret = TNC_RESULT_SUCCESS;

    if (this->count == 0) {
    	/* first package */
	    tncs.sendMessage((unsigned char*)("IMCHostScanner active"), 22, 1234567890);

    	std::stringstream send;

    	for (std::vector<Policy>::iterator iter = this->policys.begin();
    	    iter != this->policys.end();
    	    ++iter) {
    		send << (iter->protocol==TCP?"TCP":"UDP") << " " << std::dec << iter->port << std::endl;
    	}

    	std::string sendString = send.str();

    	log.info("send: \n" + sendString);
    	this->tncs.sendMessage((unsigned char*)sendString.c_str(), sendString.size(), TNC_MESSAGETYPE_FHH_HOSTSCANNER);
    } else if (this->count == 1) {
    	/* second package */
    	std::stringstream ss;

    	ss.write((const char *)message, messageLength);

	    std::vector<std::pair<std::string,std::string> > properties = TNCUtil::io::readAllProperties(&ss);

    	std::vector<Policy> policys;

		for (std::vector<std::pair<std::string,std::string> >::iterator iter = properties.begin();
		    iter != properties.end();
		    ++iter) {

			/* read Protocol */
			Protocol protocol;
			if (!iter->first.compare(0, 3, "TCP"))
				protocol = TCP;
			else if (!iter->first.compare(0, 3, "UDP"))
				protocol = UDP;
			else
				return this->clientError(log, "Protocol is unknown (not TCP or UDP): %s", iter->first.c_str());

			/* read Port */
			size_t tmpPos = iter->first.find_first_of("1234567890", 3);
			if (tmpPos==std::string::npos)
				return this->clientError(log, "No Portnumber found: %s", iter->first.c_str());

			char * end = NULL;
			unsigned long int port = strtoul(iter->first.substr(tmpPos, iter->first.size()).c_str(), &end, 10);
			if (errno==ERANGE || errno==EINVAL || *end)
				return this->clientError(log, "Error by string to unsigned int: %s", iter->first.c_str());

			/* read Status */
			Status status;
			if (!iter->second.compare("open"))
				status = open;
			else if (!iter->second.compare("close"))
				status = close;
			else if (!iter->second.compare("whatever"))
				status = whatever;
			else
				return this->clientError(log, "Status is unknown (not open, close or whatever): %s", iter->second.c_str());

			policys.push_back(Policy(protocol, (unsigned short) port, status));
		}

		for (std::vector<Policy>::iterator iter = this->policys.begin();
		    iter != this->policys.end();
		    ++iter) {
			for (std::vector<Policy>::iterator iterC = policys.begin();
			    iterC != policys.end();
			    ++iterC) {
				if (iter->protocol ==iterC->protocol &&
					iter->port == iterC->port &&
					iter->status != whatever) {
					if (iter->status != iterC->status) {
						this->actionRecommendation = TNC_IMV_ACTION_RECOMMENDATION_NO_ACCESS;
						this->evaluationResult = TNC_IMV_EVALUATION_RESULT_NONCOMPLIANT_MAJOR;
						this->validationFinished = true;
//						log.warn(formatString, va);
						this->tncs.provideRecommendation(this->actionRecommendation, this->evaluationResult);
						return TNC_RESULT_SUCCESS;
					}
				}
			}
		}
		this->actionRecommendation = TNC_IMV_ACTION_RECOMMENDATION_ALLOW;
		this->evaluationResult = TNC_IMV_EVALUATION_RESULT_COMPLIANT;
		this->validationFinished = true;
//						log.warn(formatString, va);
		this->tncs.provideRecommendation(this->actionRecommendation, this->evaluationResult);
		return TNC_RESULT_SUCCESS;

    }

    return ret;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                  TNC_Result IMVHostScanner::batchEnding()                 *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result IMVHostScanner::batchEnding()
{
	this->count++;

	return TNC_RESULT_SUCCESS;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *    TNC_Result IMVHostScanner::clientError(TNCLog &, const char*, ...)     *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result IMVHostScanner::clientError(TNCLog &log, const char * formatString, ...)
{
	this->actionRecommendation = TNC_IMV_ACTION_RECOMMENDATION_NO_RECOMMENDATION;
	this->evaluationResult = TNC_IMV_EVALUATION_RESULT_ERROR;
	this->validationFinished = true;
	va_list va;
	va_start(va, formatString);
	log.warn(formatString, va);
	va_end(va);
	return TNC_RESULT_OTHER;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *  IMVHostScanner::Policy::Policy(Protocol pr, unsigned short po, Status s) *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
IMVHostScanner::Policy::Policy(Protocol pr, unsigned short po, Status s)
:protocol(pr), port(po), status(s)
{}
