/*
 * This software is Copyright (C) 2006-2008
 * Fachhochschule Hannover (University of Applied Sciences and Arts)
 * Use is subject to license conditions.
 *
 * The main licensing options available are:
 *
 * Open Source Licensing. This is the appropriate option if you want to
 * share the source code of your application with everyone you
 * distribute it to, and you also want to give them the right to share
 * who uses it. If you wish to use TNC@FHH under Open Source Licensing,
 * you must contribute all your source code to the open source
 * community in accordance with the GPL Version 2 when your application
 * is distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * Commercial Licensing. This is the appropriate option if you are
 * creating proprietary applications and you are not prepared to
 * distribute and share the source code of your application.
 * Contact tnc@inform.fh-hannover.de for details.
 * 
 * http://tnc.inform.fh-hannover.de/
 */

#ifndef _IMVHOSTSCANNER_H_
#define _IMVHOSTSCANNER_H_

#include <TNCUtil/tcg/tnc/tncifimv.h>
/* TNCUtil */
#include <TNCUtil/iml/AbstractIMV.h>

#define TNC_VENDORID_FHH ((TNC_VendorID) 0xffff00)
#define TNC_SUBTYPE_FHH_HOSTSCANNER ((TNC_MessageSubtype) 0x30)

#define TNC_MESSAGETYPE_FHH_HOSTSCANNER (TNCUtil::formMessageType(TNC_VENDORID_FHH, TNC_SUBTYPE_FHH_HOSTSCANNER))

#define MESSAGE_TYPE_COUNT 1

#ifndef POLICY_FILE
#define POLICY_FILE "/etc/tnc/imv_hostscanner.policy"
#endif

/**
 * Header file for IMV HostScanner. This IMV checks the scan for open ports
 * done by the corresponding IMC on the client.
 *
 * <h3>Changelog:</h3>
 * <ul>
 *   <li>06.05.2008 - create class (Daniel Wuttke)</li>
 *   <li>11.02.2008 - redesign (ib)</li>
 *   <li>09.05.2008 - redesign (mbs)</li>
 * </ul>
 *
 * @class IMVHostScanner
 * @brief This IMV ckecks a clients' open ports.
 * @date 06.05.2008
 * @author Daniel Wuttke
 * @author Ingo Bente (ib)
 */
class IMVHostScanner : public TNCUtil::iml::AbstractIMV
{
    private:
    	enum Protocol {TCP, UDP};
    	enum Status {open, close, whatever};
    	
    	class Policy
    	{
    		public:
    			Protocol protocol;
    			unsigned short port;
    			Status status;
    			
    			Policy(Protocol pr, unsigned short po, Status s);
    	};
    	
    	std::vector<Policy> policys;
    	
    	unsigned int count;

    public:
    	/**
		 * Constructor
		 *
		 * @param conID - The connection ID passed from the server.
		 */
	    IMVHostScanner(TNC_ConnectionID conID);
	    
	    /**
		 * Destructor
		 */
	    virtual ~IMVHostScanner();

	    /**
		 * TODO_Comment
		 * 
		 * @see AbstractIMUnit::receiveMessage()
		 *
		 * @param enclosing_method_arguments
		 *
		 * @return return_type
		 */
        virtual TNC_Result receiveMessage(TNC_BufferReference message,
        			                      TNC_UInt32 messageLength,
        			                      TNC_MessageType messageType);
        
        virtual TNC_Result notifyConnectionChange();
        
    	virtual TNC_Result batchEnding();
    	
    	virtual TNC_Result clientError(TNCLog &log, const char * formatString, ...);
};

#endif //_IMVHOSTSCANNER_H_
